###*
* @class Tent.DateRangeField
* @extends Tent.TextField
* 
* This widget wraps the Filament Date Range Picker control. The selected value will consist of
* two dates which are bound to the {@link #startDate} and {@link #endDate} properties. 
* The {@link #value} property is also bound with the string value of the range, as seen in the 
* input control ('date1 - date2').
*
* The initial value can be sourced from the value property if provided. If no value is provided,
* then the startDate and endDate properties will be used to initialize the control.
* 
* Usage
*       {{view Tent.DateRangeField label="" 
			valueBinding="" 
			startDateBinding=""
			endDateBinding=""
			showOtherMonths=true  
			dateFormat=""
				 }}
###

require '../template/date_range'
require '../mixin/fuzzy_date_support'
require '../mixin/jquery_ui'
require '../mixin/constants'

Tent.DateRangeField = Tent.TextField.extend Tent.FuzzyDateSupport,
	templateName: 'date_range'
	classNames: ['tent-date-range-field']
	classNameBindings: ['allowFuzzyDates']
	###*
	* @property {Array} presetRanges Array of objects to be made into menu range presets. 
	* 
	* Each object requires 3 properties:
	* - text: string, text for menu item
	* - dateStart: date.js string, or function which returns a date object, start of date range
	* - dateEnd: date.js string, or function which returns a date object, end of date range
	###
	presetRanges: null

	###*
	* @property {Array} 
	* Available options are: 
	* - 'specificDate'
	* - 'allDatesBefore'
	* - 'allDatesAfter'
	* - 'dateRange'. 
	*
	* Each can be passed a string for link and label text. (example: presets: {specificDate: 'Pick a date'} )
	###
	presets: null
	
	###*
	* @property {String} rangeSplitter The character to use between two dates in the range
	###
	rangeSplitter: ','

	###*
	* @property {Date} earliestDate The earliest date allowed in the system. e.g. the 'All Dates Before'
	* range will use this as the first date in the range 
	###
	earliestDate: null

	###*
	* @property {Date} latestDate The latest date allowed in the system. e.g. the 'All Dates After'
	* range will use this as the last date in the range 
	###
	latestDate: null

	###*
	* @property {Boolean} closeOnSelect will close the rangepicker when a full range is selected
	###
	closeOnSelect: false

	###*
	* @property {Boolean} arrows will add date range advancing arrows to input.
	###
	arrows: true

	###*
	* @property {Date} startDate The selected start date in the range
	###
	startDate: null

	###*
	* @property {Date} endDate The selected end date in the range
	###
	endDate: null

	###*
	* @property {String} dateFormat The expected format for each date in the range
	###
	dateFormat: Tent.Formatting.date.getFormat()

	# This control will display a range UI, so there is no need for the template to handle that.
	hasOwnRangeDisplay: true

	operators: null # We don't need operators with a date range

	isNotTextDisplay: Ember.computed.not('textDisplay')


	defaultPresetRanges: [
		{
			text: 'Today'
			dateStart: 'today'
			dateEnd: 'today'
		}
		{
			text: 'Tomorrow'
			dateStart: 'Tomorrow'
			dateEnd: 'Tomorrow' 
		}
		{
			text: 'Last 7 days'
			dateStart: 'today-7days'
			dateEnd: 'today'
		}
		{
			text: 'Month to date'
			dateStart: ->
				return Date.parse('today').moveToFirstDayOfMonth()
			dateEnd: 'today' 
		}
		{
			text: 'Year to date'
			dateStart: ->
				x = Date.parse('today')
				x.setMonth(0)
				x.setDate(1)
				return x
			dateEnd: 'today' 
		}
		{
			text: 'The previous Month'
			dateStart: ->
				return Date.parse('1 month ago').moveToFirstDayOfMonth()
			dateEnd: ->
				return Date.parse('1 month ago').moveToLastDayOfMonth()
		}
		{
			text: 'Last 30 Days'
			dateStart: 'Today-30'
			dateEnd: 'Today' 
		}
		{	
			text: 'Next 30 Days'
			dateStart: 'Today'
			dateEnd: 'Today+30'
		}
	]

	init: ->		 
		@_super()
	
	didInsertElement: ->
		@_super(arguments)
		@setPresetRanges()
		@attachPlugin()
		@initializeValue()
		@handleReadonly()
		@handleDisabled()
		@set('filterOp', Tent.Constants.get('OPERATOR_RANGE'))

	valueForMandatoryValidation: (->
		@get('dateValue')
	).property('dateValue')

	setPresetRanges: ->
		@set('pluginPresetRanges',  $.merge(@get('defaultPresetRanges'), @get('presetRanges') || []))

	attachPlugin: ->
		widget = @
		@set('dropdownId', @get('elementId') + "dropdown")
		@set('plugin', @$('input').daterangepicker({
				id: @get('dropdownId')
				presetRanges: @get('pluginPresetRanges')
				presets: @get('presets') if @get('presets')?
				rangeSplitter: @get('rangeSplitter') if @get('rangeSplitter')?
				dateFormat: @get('dateFormat')
				earliestDate: @get('earliestDate') if @get('earliestDate')?
				latestDate: @get('latestDate') if @get('latestDate')?
				closeOnSelect: @get('closeOnSelect')
				arrows: @get('arrows')
				allowFuzzyDates: @get('allowFuzzyDates')
				datepickerOptions: {
					dateFormat : @get('dateFormat')
				}
				onChange: ->
					widget.change()
			})
		)
		@listenForFuzzyCheckboxChanges()
		@listenForFuzzyDropdownChanges()

	willDestroyElement: ->
		if not this.isDestroyed
			@$('input').remove()


	###*
	* @method getValue Return the current value of the input field
	* @return {String}
	###
	getValue: ->
		@$('.ember-text-field').val() if @$('.ember-text-field')?

	###*
	* @method setValue Set the value of the input field
	* @param {String} value
	###
	setValue: (value)->
		@$('.ember-text-field').val(value)

	initializeValue: ->
		if not @get('value')? and not @get('fuzzyValue')?
			@setValue(@getDateStringFromStartAndEndDates())
		if @get('fuzzyValue')?
			@initializeFromFuzzyValue()
		else
			@resetFuzzyValue()
			
	getDateStringFromStartAndEndDates: ->
		if @get('startDate')?
			start = Tent.Formatting.date.format(@get('startDate'), @get('dateFormat'))
		if @get('endDate')?
			end = Tent.Formatting.date.format(@get('endDate'), @get('dateFormat'))
		start + @get('rangeSplitter') + " " + end

	placeholder: (->
		@get('dateFormat') + @get('rangeSplitter') + " " + @get('dateFormat')
	).property('dateFormat')

	change: (e)->
		if e? and not $(e.originalTarget).is('.useFuzzy')
			return

		if not @isFuzzyDate(@getValue())
			@set('dateValue', @getValue())
		else 
			@set('dateValue', @getDateStringFromFuzzyValue(@getValue()))

		@set("fuzzyValueTemp", @getValue())
		@set('isValid', @validate())
		if @get('isValid')
			unformatted = @unFormat(@get('dateValue'))
			@set('value', @convertSingleDateToDateRange(unformatted))

	focusOut: ->
		# override the default behavior for textfields, since we do not want to validate
		# for just interacting with the dropdown.

	convertSingleDateToDateRange: (date)->
		if @isFuzzyDate(date)
			return date
		if (date.indexOf(",") == -1)
			date += ",#{date}"
		date

	validate: ->
		isValid = @_super()
		isValidStartDate = isValidEndDate = true
		if @get('dateValue')? and @get('dateValue') != ""
			startString = @getStartFromDate(@get('dateValue'))
			if startString?
				try 
					startDate = Tent.Formatting.date.unformat(startString.trim(), @get('dateFormat'))
					@set('startDate', startDate)
				catch e
					isValidStartDate = false
					@set('startDate', null)

			endString = @getEndFromDate(@get('dateValue'))
			if endString?
				try
				 endDate = Tent.Formatting.date.unformat(endString.trim(), @get('dateFormat'))
				 @set('endDate', endDate)
				catch e
				 isValidEndDate = false
				 @set('endDate', null)
			else
				@set('endDate', @get('startDate'))
				
		@addValidationError(Tent.messages.DATE_FORMAT_ERROR) unless ((isValidStartDate and isValidEndDate) or @isFuzzyDateValid(@get('formattedValueTemp')))
		@validateWarnings() if (@isFuzzyDateValid(@get('formattedValueTemp')) or (isValid and isValidStartDate and isValidEndDate))
		@isFuzzyDateValid(@get('formattedValueTemp')) || (isValid && isValidStartDate && isValidEndDate)

	getStartFromDate: (date) ->
		date.split(@get('rangeSplitter'))[0]

	getEndFromDate: (date) ->
		date.split(@get('rangeSplitter'))[1]

	validateWarnings: ->
		@_super()

	#Format for display
	format: (value)->
		value

	# Format for binding
	unFormat: (value)->
		value

	readOnlyHandler: (e)->
		e.preventDefault()
		e.stopPropagation()
		$('.ui-daterangepicker').hide()
		return false
	
	handleReadonly: (->
		if @get('readOnly')? && @get('readOnly')
			@$('.ui-rangepicker-input').bind('click', @get('readOnlyHandler'))
			@$('.ui-daterangepicker-prev, .ui-daterangepicker-next').css("visibility", "hidden")
		else 
			@$('.ui-rangepicker-input, .ui-daterangepicker-prev, .ui-daterangepicker-next').unbind('click', @get('readOnlyHandler'))
			@$('.ui-daterangepicker-prev, .ui-daterangepicker-next').css("visibility", "visible")
	).observes('readOnly')

	handleDisabled: (->
		if @$()?
			if @get('disabled')? && @get('disabled')
				@.$('.ui-daterangepicker-prev, .ui-daterangepicker-next').css("visibility", "hidden")
			else 
				@.$('.ui-daterangepicker-prev, .ui-daterangepicker-next').css("visibility", "visible")
	).observes('disabled')



 
